"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AuthenticatorFactory = void 0;
const jsonwebtoken_1 = require("jsonwebtoken");
const generated_src_1 = require("../generated-src");
class AuthenticatorFactory {
    // eslint-disable-next-line no-useless-constructor
    constructor(cacheTtl = AuthenticatorFactory.DEFAULT_CACHE_TTL, api = new generated_src_1.UnauthenticatedApi()) {
        this.cacheTtl = cacheTtl;
        this.api = api;
    }
    create(request) {
        let token;
        const api = this.api;
        const getToken = () => __awaiter(this, void 0, void 0, function* () {
            if (!token) {
                token = (yield api.login(request)).data.jwt;
                setTimeout(() => {
                    token = undefined;
                }, this.cacheTtl);
            }
            return token;
        });
        return {
            getToken,
            getPayload: () => __awaiter(this, void 0, void 0, function* () {
                return jsonwebtoken_1.decode(yield getToken());
            }),
        };
    }
}
exports.AuthenticatorFactory = AuthenticatorFactory;
// eslint-disable-next-line @typescript-eslint/no-magic-numbers
AuthenticatorFactory.DEFAULT_CACHE_TTL = 24 * 60 * 60 * 1000; // 1 day
//# sourceMappingURL=authenticator.js.map