"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.LocationListener = exports.ConnectionState = exports.LocationEventType = void 0;
const events_1 = require("events");
const pusher_js_1 = __importDefault(require("pusher-js"));
const generated_src_1 = require("../generated-src");
const base_1 = require("../generated-src/base");
const API_KEY = 'baf06f5a867d462e09d4';
const AUTH_ENDPOINT = base_1.BASE_PATH + '/auth/pusher';
const CONNECTION_STATE_EVENT = 'state_change';
var LocationEventType;
(function (LocationEventType) {
    LocationEventType["ConnectionState"] = "connection_state";
    LocationEventType["DeviceAlarm"] = "device_alarm";
    LocationEventType["DevicePair"] = "device_pair";
    LocationEventType["DeviceTrigger"] = "device_trigger";
    // eslint-disable-next-line @typescript-eslint/no-shadow
    LocationEventType["Hub"] = "hub";
    LocationEventType["Mode"] = "mode";
    LocationEventType["Rfid"] = "rfid";
})(LocationEventType = exports.LocationEventType || (exports.LocationEventType = {}));
var ConnectionState;
(function (ConnectionState) {
    ConnectionState["Connecting"] = "connecting";
    ConnectionState["Connected"] = "connected";
    ConnectionState["Disconnected"] = "disconnected";
    ConnectionState["Failed"] = "failed";
    ConnectionState["Unavailable"] = "unavailable";
})(ConnectionState = exports.ConnectionState || (exports.ConnectionState = {}));
class LocationListener {
    // eslint-disable-next-line no-useless-constructor
    constructor(authenticator) {
        this.authenticator = authenticator;
        this.eventEmitter = new events_1.EventEmitter();
        this.connectionState = ConnectionState.Disconnected;
    }
    getConnectionState() {
        return this.connectionState;
    }
    connect() {
        return __awaiter(this, void 0, void 0, function* () {
            const auth = yield this.getAuthConfig();
            if (this.pusher) {
                this.pusher.config.auth = auth;
                this.pusher.connect();
            }
            else {
                this.pusher = new pusher_js_1.default(API_KEY, {
                    authEndpoint: AUTH_ENDPOINT,
                    auth,
                });
                this.pusher.connection.bind(CONNECTION_STATE_EVENT, (event) => this.emitConnectionStateEvent(event));
            }
        });
    }
    disconnect() {
        var _a;
        (_a = this.pusher) === null || _a === void 0 ? void 0 : _a.disconnect();
    }
    addLocation(locationId) {
        if (!this.pusher) {
            throw new Error('The listener is not connected, likely due to a missing call to connect().');
        }
        const channelName = `private-${locationId}`;
        let channel = this.pusher.channel(channelName);
        if (!channel) {
            channel = this.pusher.subscribe(channelName);
        }
        channel.bind(generated_src_1.EventType.Device, (event) => this.emitDeviceEvent(event, locationId));
        channel.bind(generated_src_1.EventType.Hub, (event) => this.emit(LocationEventType.Hub, event, locationId));
        channel.bind(generated_src_1.EventType.Mode, (event) => this.emit(LocationEventType.Mode, event, locationId));
        channel.bind(generated_src_1.EventType.Rfid, (event) => this.emit(LocationEventType.Rfid, event, locationId));
    }
    removeLocation(locationId) {
        if (!this.pusher) {
            throw new Error('The listener is not connected, likely due to a missing call to connect().');
        }
        const channelName = `private-${locationId}`;
        const channel = this.pusher.channel(channelName);
        if (channel) {
            this.pusher.unsubscribe(channelName);
        }
    }
    on(eventType, listener) {
        this.eventEmitter.on(eventType, listener);
    }
    off(eventType, listener) {
        this.eventEmitter.off(eventType, listener);
    }
    getAuthConfig() {
        return __awaiter(this, void 0, void 0, function* () {
            return {
                headers: {
                    Authorization: yield this.authenticator.getToken(),
                },
            };
        });
    }
    emitConnectionStateEvent(event) {
        this.connectionState = event.current;
        this.eventEmitter.emit(LocationEventType.ConnectionState, event);
    }
    emitDeviceEvent(event, locationId) {
        switch (event.event) {
            case generated_src_1.DeviceEventType.Alarmed:
            case generated_src_1.DeviceEventType.Dismissed:
                this.emit(LocationEventType.DeviceAlarm, event, locationId);
                break;
            case generated_src_1.DeviceEventType.Paired:
            case generated_src_1.DeviceEventType.Unpaired:
                this.emit(LocationEventType.DevicePair, event, locationId);
                break;
            case generated_src_1.DeviceEventType.Triggered:
                this.emit(LocationEventType.DeviceTrigger, event, locationId);
                break;
        }
    }
    emit(type, event, locationId) {
        this.eventEmitter.emit(type, event, locationId);
    }
}
exports.LocationListener = LocationListener;
//# sourceMappingURL=locationListener.js.map